// Test for Resample.hxx
// basic compile might look as follows:
// g++ ResampleTest.cc -I/home/maciek/opt/include -I../Ifg2Spectrum -L/home/maciek/opt/lib -lfftw -lfftw_threads -lpthread

#include <iostream>
#include <sstream>
#include <vector>
#include <boost/array.hpp>

#include "Physics.h"
#include "Apodize.hxx"
#include <netcdfcpp.h>

using namespace std;

const static int LW_SPECTRUM_SIZE=1024;
const static int MW_SPECTRUM_SIZE=2048;

typedef boost::array<float, LW_SPECTRUM_SIZE> LwData;
typedef boost::array<float, MW_SPECTRUM_SIZE> MwData;
typedef gips::Apodize<LwData, LwData> LwApodize;
typedef gips::Apodize<MwData, MwData> MwApodize;

void apodizeCube(const string& cubein, const string& cubeout)
{
        NcFile nfi(cubein.c_str(), NcFile::ReadOnly);
        NcVar* isignalRealLw(nfi.get_var("signalReal_lw"));
        NcVar* isignalRealMw(nfi.get_var("signalReal_mw"));
        
        NcFile nfo(cubeout.c_str(), NcFile::Replace);
        NcDim* ospectralDimLw(nfo.add_dim("npts_lw", 1024));
        NcDim* ospectralDimMw(nfo.add_dim("npts_mw", 2048));
        NcDim* orowDim(nfo.add_dim("row_indx", 128));
        NcDim* ocolDim(nfo.add_dim("col_indx", 128));
        NcVar* osignalRealLw(nfo.add_var("signalReal_lw", ncFloat, orowDim, ocolDim, ospectralDimLw));
        NcVar* osignalRealMw(nfo.add_var("signalReal_mw", ncFloat, orowDim, ocolDim, ospectralDimMw));

        LwData unapodlw, apodlw;
        MwData unapodmw, apodmw;

        LwApodize lwapodize(LwApodize::Ports(&unapodlw, &apodlw));
        MwApodize mwapodize(MwApodize::Ports(&unapodmw, &apodmw));
        
        for (int i=0; i < 128; ++i)
        {
            for (int j=0; j < 128; ++j)
            {
                isignalRealLw->set_cur(i,j,0);
                isignalRealMw->set_cur(i,j,0);
                isignalRealLw->get(&unapodlw[0], 1, 1, LW_SPECTRUM_SIZE);
                isignalRealMw->get(&unapodmw[0], 1, 1, MW_SPECTRUM_SIZE);


                lwapodize(); 
                mwapodize();

                osignalRealLw->set_cur(i,j,0);
                osignalRealMw->set_cur(i,j,0);
                osignalRealLw->put(&apodlw[0], 1, 1, LW_SPECTRUM_SIZE);
                osignalRealMw->put(&apodmw[0], 1, 1, MW_SPECTRUM_SIZE);
            }
            std::cerr << ".";
        }
        std::cerr << std::endl;
}
        
int usage(string execname)
{
    cerr << "USAGE:" << endl;
    cerr << execname << " infile outfile" << endl;
    return 1;
}

int main(int argc, char **argv)
{
    if (argc != 3)
        return usage(argv[0]);

    apodizeCube(argv[1], argv[2]);
    return 0;
}
