/**
 * \file IfgShift.hxx
 * \brief Shifts raw interferogram to form expected by the FFT operator.
 * \author Maciek Smuga-Otto <maciek@ssec.wisc.edu>
 *
 * based directly on the Scanning-HIS rsh2fbf application, written by Ray Garcia <rayg@ssec.wisc.edu>
 *
 *  \version $Id: IfgShift.hxx,v 1.10.4.2 2006/03/29 16:53:44 maciek Exp $
 *
 *  \par Copyright:
 *  \verbatim
 *
 *  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
 *  Space Science and Engineering Center
 *  University of Wisconsin - Madison, USA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *  \endverbatim
 */



#ifndef H_IFGSHIFT
#define H_IFGSHIFT

#include <complex>

namespace gips {

/*!  rotate vector halfway around, bringing endpoints to center and center to endpoints.
  *
  *  \par Restrictions:
  *  InType's elements must copy to OutType container
  *
  */
template< typename InType, typename OutType >
class IfgShift
{    
public:
    struct Ports
    {
        const InType *obs_in;
        OutType *obs_out;
        
        Ports(const InType *in_, OutType *out_ ): obs_in(in_), obs_out(out_) { }
    };

private:
    Ports ports;    

public:
    
    IfgShift(const Ports &p): ports(p) { }
    
    // The meat of the template. 
    void operator() ( )
    {
        const InType& in(*ports.obs_in);
        OutType& out(*ports.obs_out);

        unsigned halfN(InType::static_size/2); 

        std::copy(in.begin() + halfN, in.end(), out.begin());
        std::copy(in.begin(), in.begin() + halfN, out.begin() + halfN ); 
    } 
}; // class 

} // namespace

#endif
    
