/**
 * \file SpectrumChop.h
 * \author Maciek Smuga-Otto <maciek@ssec.wisc.edu>
 *
 *  \version $Id: SpectrumChop.hxx,v 1.9.4.1 2006/02/27 21:20:33 graemem Exp $
 *
 *  \par Copyright:
 *  \verbatim
 *
 *  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
 *  Space Science and Engineering Center
 *  University of Wisconsin - Madison, USA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  \endverbatim
 */

#ifndef H_SPECCHOP
#define H_SPECCHOP

#include <complex>

// NOTE: This file is an example of the kind of output expected from the 
// GIPS flanging compiler, and will in the future NOT be generated by hand.
// As a result, all the input/output structs are Plain Old Data (POD)s,
// as indicated by the _t suffixes. Furthermore, all their members have
// names such as would be published in the global data dictionary.


namespace gips 
{

template
<
    typename InType,
    typename OutType
>   
class SpectrumChop
{
public:
    struct settings_t 
    {
        unsigned int chopRawSpectrumFrom; 
        bool wrapRawSpectrumLastPoint;

        template< typename Src > 
        static settings_t fromStruct( const Src &x )
        {
            settings_t ret;
            ret.chopRawSpectrumFrom = x.chopRawSpectrumFrom;
            ret.wrapRawSpectrumLastPoint = x.wrapRawSpectrumLastPoint;
            return ret;
        }
    };
    
    struct Ports 
    {
        const InType *obs_in;
        OutType *obs_out;
        
        explicit Ports(         
            const InType *obs_in_,
            OutType *obs_out_
            ):
               obs_in(obs_in_),
               obs_out(obs_out_)
        {
        }
    };

protected:
    const settings_t settings;
    Ports ports;
                 
public:    

    explicit SpectrumChop(const settings_t &s, const Ports &p): settings(s), ports(p)
    {    
        // check if obs_out is big enough to hold the resulting chop.
        // Note: can't replace with static assert due to runtime nature of chapRawSpectrumFrom.
        assert(ports.obs_out->size() >= ports.obs_in->size() - settings.chopRawSpectrumFrom);
    }
    
    int operator()(void)
    {    
        std::copy(
                ports.obs_in->begin() + settings.chopRawSpectrumFrom,
//                ports.obs_in->begin() + settings.chopRawSpectrumFrom + ports.obs_out->size(),
                ports.obs_in->end(),
                ports.obs_out->begin()
        );
        
        // curious case of the last point: wrap it, or leave it?
        if (settings.wrapRawSpectrumLastPoint)
            *(ports.obs_out->end() - 1) = *(ports.obs_in->begin() + settings.chopRawSpectrumFrom);
        return 0;
    }

}; // class SpectrumChop
} // namespace


#endif
