/*
 * \file SpectrumChopTest.cc
 * \brief unit test for SpectrumChop operator
 * \author Maciek Smuga-Otto <maciek@ssec.wisc.edu>
 *
 *  \version $Id: SpectrumChopTest.cc,v 1.8.6.1 2006/02/27 21:20:33 graemem Exp $
 *
 *  \par Copyright:
 *  \verbatim
 *
 *  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
 *  Space Science and Engineering Center
 *  University of Wisconsin - Madison, USA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *  \endverbatim
 */


#include <iostream>
#include <complex>
#include <boost/array.hpp>
#include "SpectrumChop.hxx"

using namespace gips;
using namespace std;
using namespace boost;

template <std::size_t INSIZE, std::size_t OUTSIZE>
class TestEnv
{
    typedef boost::array<complex<double>, INSIZE> InType;
    typedef boost::array<complex<double>, OUTSIZE> OutType;
    typedef SpectrumChop<InType, OutType> BasicSpectrumChop;

public:
    void livenessTest()
    {
        typename BasicSpectrumChop::settings_t settings;

        settings.chopRawSpectrumFrom = INSIZE - OUTSIZE;
        settings.wrapRawSpectrumLastPoint = false;

        InType  obs_in;
        OutType obs_out;

        fill(obs_in.begin(), obs_in.end(), 1.0);
        fill(obs_out.begin(), obs_out.end(), 0.0);
    
        typename BasicSpectrumChop::Ports ports(&obs_in, &obs_out);
  
        BasicSpectrumChop stage( settings, ports );
        stage();
    }
};

int main()
{
    TestEnv<2048, 1025> test;
    test.livenessTest();
    return 0;
}
