// Test for Resample.hxx
// basic compile might look as follows:
// g++ ResampleTest.cc -I/home/maciek/opt/include -I../Ifg2Spectrum -L/home/maciek/opt/lib -lfftw -lfftw_threads -lpthread

#include <iostream>
#include <sstream>
#include <vector>
#include <boost/array.hpp>

#include "Physics.h"
#include "Resample.hxx"
#include "FBF.h"

using namespace std;

enum {SPECTRUM_SIZE=1024, UPSAMPLE_SIZE=262144};

typedef boost::array<double, SPECTRUM_SIZE> Data;
typedef gips::SpectralResampling<Data, Data, UPSAMPLE_SIZE> Resampler;

void singleSpectrumTest(const string& fbfwn, const string& fbfin, const string& fbfout, double lwr)
{
    Data testwn;
    FBF::load_static(testwn, fbfwn);
    // below code looks funny to compensate for known bug in load_static, which incorrectly fails to 
    // import last point in array.
    gips::physics::WnumScale<double, SPECTRUM_SIZE> wn(testwn[0], 
                    testwn[0] + (testwn[1]-testwn[0]) * (SPECTRUM_SIZE - 1));
    Resampler::settings_t settings(wn);
    
    Resampler::ancillary_t ancil;
    ancil.laserWavenumberRatio = lwr;
    
    Data testin, testout;
    
    Resampler resample(
        settings, 
        Resampler::Ports(&ancil, &testin, &testout)
    );

    FBF::load_static(testin, fbfin);
    resample();
    FBF::save(fbfout, testout);
}

int usage(string execname)
{
    cerr << "USAGE:" << endl;
    cerr << execname << " fbfWnFile fbfInFile fbfOutFile lwRatio" << endl;
    return 1;
}

int main(int argc, char **argv)
{
    if (argc != 5)
        return usage(argv[0]);

    std::vector<int> fbfSizes, nominalSizes(3, SPECTRUM_SIZE);
    // push back dimensions of each fbf file, should be SPECTRUM_SIZE
    FBF::dimensions(fbfSizes, argv[1]);
    FBF::dimensions(fbfSizes, argv[2]);
    FBF::dimensions(fbfSizes, argv[3]);
    if (fbfSizes != nominalSizes)
    {
        cerr << "sizes of wn, in and out files must all have " 
             << SPECTRUM_SIZE << " spectral points." << endl;
        return 1;
    }
    double lwRatio;
    istringstream sin(argv[4]);
    sin >> lwRatio;
    
    singleSpectrumTest(argv[1], argv[2], argv[3], lwRatio);
    return 0;
}
